@@ -462,14 +462,22 @@ IOReturn clipAppleUSBAudioToOutputStream_old (const void *mixBuf,
 
 // aml 2.21.02 added second filter state for 4th order filter
 // aml 2.21.02 added more filter state for phase compensator
-IOReturn clipAppleUSBAudioToOutputStreamiSub (const void *mixBuf, void *sampleBuf, PreviousValues * filterState,  PreviousValues * filterState2, PreviousValues * phaseCompState, Float32 *low, Float32 *high, UInt32 firstSampleFrame, UInt32 numSampleFrames, UInt32 sampleRate, const IOAudioStreamFormat *streamFormat, SInt16 *iSubBufferMemory, UInt32 *loopCount, SInt32 *iSubBufferOffset, UInt32 iSubBufferLen)
+ // aml 3.4.02 added srcPhase
+IOReturn clipAppleUSBAudioToOutputStreamiSub (const void *mixBuf, void *sampleBuf, PreviousValues * filterState,  PreviousValues * filterState2, PreviousValues * phaseCompState, Float32 *low, Float32 *high, UInt32 firstSampleFrame, UInt32 numSampleFrames, UInt32 sampleRate, const IOAudioStreamFormat *streamFormat, SInt16 *iSubBufferMemory, UInt32 *loopCount, SInt32 *iSubBufferOffset, UInt32 iSubBufferLen, iSubAudioFormatType* iSubFormat, float* srcPhase, float* srcState)
 {
     UInt32 sampleIndex, maxSampleIndex;
     float *floatMixBuf;
     SInt16 *outputBuf;
-	float highSample;
-	Float32	iSubSampleFloat;
-	SInt16	iSubSampleInt;
+    float highSample;
+    Float32	iSubSampleFloat;
+    SInt16	iSubSampleInt;
+
+    // aml 3.6.02 storage for src
+    float 	x0, x1, temp;
+
+    // aml 3.6.02 src variables - should calculate phaseInc somewhere else, change only if the SR changes
+    float 	phaseInc = ((float)sampleRate)/((float)(iSubFormat->outputSampleRate));	// phase increment = Fs_in/Fs_out
+    float 	phase = *srcPhase;							// current phase location
 
     floatMixBuf = (float *)mixBuf;
     outputBuf = (SInt16 *)sampleBuf;
@@ -482,36 +490,107 @@ IOReturn clipAppleUSBAudioToOutputStreamiSub (const void *mixBuf, void *sampleBu
 	} else if (2 == streamFormat->fNumChannels) {
 		// aml 2.21.02 changed to 4th order version
 		// aml 2.21.02 changed to 4th order version with phase compensation
-                StereoFilter4thOrderPhaseComp (&floatMixBuf[firstSampleFrame * streamFormat->fNumChannels], &low[firstSampleFrame * streamFormat->fNumChannels], &high[firstSampleFrame * streamFormat->fNumChannels], numSampleFrames, sampleRate, filterState, filterState2, phaseCompState);
-	}
+            StereoFilter4thOrderPhaseComp (&floatMixBuf[firstSampleFrame * streamFormat->fNumChannels], &low[firstSampleFrame * streamFormat->fNumChannels], &high[firstSampleFrame * streamFormat->fNumChannels], numSampleFrames, sampleRate, filterState, filterState2, phaseCompState);
+    }
 
+    for (sampleIndex = (firstSampleFrame * streamFormat->fNumChannels); sampleIndex < maxSampleIndex; sampleIndex++) {
+            highSample = high[sampleIndex];
+            if (highSample > 1.0) {
+                    highSample = 1.0;
+            } else if (highSample < -1.0) {
+                    highSample = -1.0;
+            }
+
+            outputBuf[sampleIndex] = HostToUSBWord ((SInt16)(highSample * 32767.0));
+    }
+    
+    // aml 3.01.02 adding num channels check
+    if ((iSubFormat->numChannels == 1) && (streamFormat->fNumChannels == 2)) {
+        // aml 3.6.02 linear interpolation src (takes the edge of the zoh version, without wasting too many 
+        // cycles since we have a 4th order lp in front of us, down -90 dB at Nyquist for 6kHz sample rate)
+        // STEREO->MONO
+        sampleIndex = (firstSampleFrame * streamFormat->fNumChannels);
+        while (sampleIndex < maxSampleIndex) {
+                    
+            if (phase >= 1.0) {	
+                phase -= 1.0;
+                sampleIndex+=2;
+            } else {   
+                // check for beginning of frame case, use saved last sample if needed
+                if (sampleIndex == (firstSampleFrame * streamFormat->fNumChannels)) { 
+                    x0 = *srcState;
+                } else {
+                    // mix x[n-1] to mono
+                    x0 = low[sampleIndex-2];
+                    temp = low[sampleIndex-1];
+                    x0 = 0.5*(x0 + temp);
+                }
+                
+                // mix x[n] to mono
+                x1 = low[sampleIndex];
+                temp = low[sampleIndex+1];
+                x1 = 0.5*(x1 + temp);
+                                
+                // linearly interpolate between x0 and x1
+                iSubSampleFloat = x0 + phase*(x1 - x0);
+                
+                // clip
+                if (iSubSampleFloat > 1.0) {
+                    iSubSampleFloat = 1.0;
+                } else if (iSubSampleFloat < -1.0) {
+                    iSubSampleFloat = -1.0;
+                }
+                
+                // convert to fixed
+                if (iSubSampleFloat >= 0) {
+                    iSubSampleInt = (SInt16) (iSubSampleFloat * 32767.0);
+                } else {
+                    iSubSampleInt = (SInt16) (iSubSampleFloat * 32768.0);
+                }
+                
+                // check for end of buffer condition
+                if (*iSubBufferOffset >= (SInt32)iSubBufferLen) {
+                        *iSubBufferOffset = 0;
+                        (*loopCount)++;
+                }
+                
+                // byteswap to USB format and copy to iSub buffer
+                iSubBufferMemory[(*iSubBufferOffset)++] = ((((UInt16)iSubSampleInt) << 8) & 0xFF00) | ((((UInt16)iSubSampleInt) >> 8) & 0x00FF);
+
+                // increment phase and update input buffer pointer
+                phase += phaseInc;		
+            }
+	}
+        if (phase < 1) {
+            // mix and save last sample in buffer to mono if it will be needed for the next loop
+            x1 = low[maxSampleIndex-2];
+            temp = low[maxSampleIndex-1];
+            *srcState = 0.5*(x1 + temp);
+        } else {
+            *srcState = 0;
+        }
+        // cache current phase for use next time we enter the clip loop
+       *srcPhase = phase;
+    } else {
+        // STEREO->STEREO, MONO->MONO
 	for (sampleIndex = (firstSampleFrame * streamFormat->fNumChannels); sampleIndex < maxSampleIndex; sampleIndex++) {
-		highSample = high[sampleIndex];
-		if (highSample > 1.0) {
-			highSample = 1.0;
-		} else if (highSample < -1.0) {
-			highSample = -1.0;
-		}
-
-		outputBuf[sampleIndex] = HostToUSBWord ((SInt16)(highSample * 32767.0));
-
-		iSubSampleFloat = low[sampleIndex];
-		if (iSubSampleFloat > 1.0) {
-			iSubSampleFloat = 1.0;
-		} else if (iSubSampleFloat < -1.0) {
-			iSubSampleFloat = -1.0;
-		}
-
-		iSubSampleInt = (SInt16) (iSubSampleFloat * 32767.0);
-
-		if (*iSubBufferOffset >= (SInt32)iSubBufferLen) {
-			*iSubBufferOffset = 0;
-			(*loopCount)++;
-		}
-
-		iSubBufferMemory[(*iSubBufferOffset)++] = ((((UInt16)iSubSampleInt) << 8) & 0xFF00) | ((((UInt16)iSubSampleInt) >> 8) & 0x00FF);
-	}
-
+            iSubSampleFloat = low[sampleIndex];
+            if (iSubSampleFloat > 1.0) {
+                    iSubSampleFloat = 1.0;
+            } else if (iSubSampleFloat < -1.0) {
+                    iSubSampleFloat = -1.0;
+            }
+    
+            iSubSampleInt = (SInt16) (iSubSampleFloat * 32767.0);
+     
+            if (*iSubBufferOffset >= (SInt32)iSubBufferLen) {
+                    *iSubBufferOffset = 0;
+                    (*loopCount)++;
+            }
+    
+            iSubBufferMemory[(*iSubBufferOffset)++] = ((((UInt16)iSubSampleInt) << 8) & 0xFF00) | ((((UInt16)iSubSampleInt) >> 8) & 0x00FF);
+        }
+    }
     return kIOReturnSuccess;
 }
 
