@@ -48,15 +48,29 @@ bool AppleUSBAudioLevelControl::init (UInt8 theUnitID, UInt8 theInterfaceNumber,
 	IOFixed						resolutionDB;
 	IOReturn					ret;
 	Boolean						result;
+	Boolean						arePRAMControl;
 
 	debug8IOLog ("+AppleUSBAudioLevelControl[%p]::init (%d, %d, %d, %d, %p, %p)\n", this, theUnitID, theInterfaceNumber, theControlSelector, theChannelNumber, theUSBDeviceRequest, properties);
 	result = FALSE;
-	FailIf (NULL == theUSBDeviceRequest, Exit);
-	FailIf (VOLUME_CONTROL != theControlSelector, Exit);	 // Only supports volume control for now - add more later
+	arePRAMControl = FALSE;
 
+	FailIf (NULL == theUSBDeviceRequest, Exit);
 	setValueThreadCall = thread_call_allocate ((thread_call_func_t)updateValueCallback, this);
 	FailIf (NULL == setValueThreadCall, Exit);
 
+	// If this control is supposed to be a pram volume control, pretend we're just a regular volume control
+	// so that we can get the min and max dB from the device as we init
+	if (theControlSelector == 0xff) {
+		theControlSelector = VOLUME_CONTROL;
+		arePRAMControl = TRUE;
+	}
+
+	if (kIOAudioLevelControlSubTypeLFEVolume == subType) {
+		// The iSub controls are on channels 1 and 2, but we want to make them look like they're on channel 0 for the HAL
+		// set it to 1 here so we can query the iSub
+		theChannelNumber = 1;
+	}
+
 	unitID = theUnitID;
 	interfaceNumber = theInterfaceNumber;
 	controlSelector = theControlSelector;
@@ -84,16 +98,16 @@ bool AppleUSBAudioLevelControl::init (UInt8 theUnitID, UInt8 theInterfaceNumber,
 	}
 
 	currentValue = GetCurVolume (interfaceNumber, channelNumber, &ret);
-    FailIf (kIOReturnSuccess != ret, Exit);
+	FailIf (kIOReturnSuccess != ret, Exit);
 	debug3IOLog ("channelNumber %d, currentValue = 0x%X, ", channelNumber, currentValue);
 	volRes = GetVolumeResolution (interfaceNumber, channelNumber, &ret);
-    FailIf (kIOReturnSuccess != ret, Exit);
+	FailIf (kIOReturnSuccess != ret, Exit);
 	debug2IOLog ("vol res = %d, ", volRes);
 	deviceMin = GetMinVolume (interfaceNumber, channelNumber, &ret);
-    FailIf (kIOReturnSuccess != ret, Exit);
+	FailIf (kIOReturnSuccess != ret, Exit);
 	debug2IOLog ("deviceMin = 0x%X, ", deviceMin);
 	deviceMax = GetMaxVolume (interfaceNumber, channelNumber, &ret);
-    FailIf (kIOReturnSuccess != ret, Exit);
+	FailIf (kIOReturnSuccess != ret, Exit);
 	debug2IOLog ("deviceMax = 0x%X\n", deviceMax);
 
 	// Having the device say that it does -infinity dB messes up our math, so set the min at -127.9961dB instead.
@@ -109,23 +123,43 @@ bool AppleUSBAudioLevelControl::init (UInt8 theUnitID, UInt8 theInterfaceNumber,
 	offset = -deviceMin;
 	debug2IOLog ("offset = %d\n", offset);
 
-/*	The device won't ever report itself being outside of its allowable range -- that would be silly.
-	if (currentValue < deviceMin) {
-		currentValue = deviceMin;
-	} else if (currentValue > deviceMax) {
-		currentValue = deviceMax;
-	} */
-
 	currentValue = (currentValue + offset) / volRes;
 	deviceMax = ((deviceMin + offset) + (deviceMax + offset)) / volRes;
 
 	// Set values needed to compute proper PRAM boot beep volume setting
 	fMaxVolume = deviceMax;
 	fMinVolume = deviceMin + offset;
+	deviceMin = -1;
 
-//	FailIf (!super::init (currentValue + offset, deviceMin + offset, deviceMax + offset, deviceMinDB, deviceMaxDB, theChannelNumber, channelName, 0, subType, usage), Exit);
-	debug4IOLog ("min = %d, max = %d, current = %d\n", (deviceMin + offset) - 1, deviceMax, currentValue);
-	FailIf (FALSE == super::init (currentValue, (deviceMin + offset) - 1, deviceMax, deviceMinDB, deviceMaxDB, theChannelNumber, channelName, 0, subType, usage), Exit);
+	if (arePRAMControl) {
+		// If this is a 'pram' control then there is no need to call the hardware.
+		UInt8 						curPRAMVol;
+		IODTPlatformExpert * 		platform = NULL;
+
+		curPRAMVol = 0;
+		platform = OSDynamicCast (IODTPlatformExpert, getPlatform ());
+		if (NULL != platform) {
+			platform->readXPRAM ((IOByteCount)kPRamVolumeAddr, &curPRAMVol, (IOByteCount)1);
+			curPRAMVol = (curPRAMVol & 0xF8);
+		}
+		currentValue = curPRAMVol;
+		deviceMin = 0;
+		deviceMax = 7;
+		subType = kIOAudioLevelControlSubTypePRAMVolume;
+		channelName = kIOAudioControlChannelNameAll;
+		theChannelNumber = 0;	// force it to the master channel even though we're piggy backing off of the channel 1 control
+	}
+
+	if (kIOAudioLevelControlSubTypeLFEVolume == subType) {
+		// The iSub controls are on channels 1 and 2, but we want to make them look like they're on channel 0 for the HAL
+		theChannelNumber = 0;
+		channelName = kIOAudioControlChannelNameAll;
+	}
+
+//	debug4IOLog ("min = %d, max = %d, current = %d\n", (deviceMin + offset) - 1, deviceMax, currentValue);
+	debug4IOLog ("min = %d, max = %d, current = %d\n", deviceMin, deviceMax, currentValue);
+
+	FailIf (FALSE == super::init (currentValue, deviceMin, deviceMax, deviceMinDB, deviceMaxDB, theChannelNumber, channelName, 0, subType, usage), Exit);
 
 	result = TRUE;
 
@@ -157,10 +191,6 @@ IOReturn AppleUSBAudioLevelControl::performValueChange (OSObject * newValue) {
 	newValueAsSInt32 = newValueAsNumber->unsigned32BitValue ();
 	debug3IOLog ("++AppleUSBAudioLevelControl[%p]::performValueChange (%ld)\n", this, newValueAsSInt32);
 
-    // updateUSBValue ();
-    // We should just be able to make an asynchronous deviceRequest, but for some reason that doesn't want to work
-    // we get pipe stall errors and the change is never made
-
     if (NULL != setValueThreadCall) {
         thread_call_enter1 (setValueThreadCall, (thread_call_param_t)newValueAsSInt32);
     }
@@ -176,7 +206,7 @@ void AppleUSBAudioLevelControl::updateUSBValue () {
 }
 
 void AppleUSBAudioLevelControl::updateUSBValue (SInt32 newValue) {
-	SInt32						newiSubVolume;
+//	SInt32						newiSubVolume;
     SInt16						theValue;
 	SInt16						newVolume;
     IOReturn					ret;
@@ -192,12 +222,17 @@ void AppleUSBAudioLevelControl::updateUSBValue (SInt32 newValue) {
 	debug2IOLog ("setting volume to 0x%X\n", newVolume);
 	ret = SetCurVolume (interfaceNumber, channelNumber, theValue);
 
+	if (getSubType () == kIOAudioLevelControlSubTypeLFEVolume) {
+		// We set the iSub's left volume (on channel 1 above), now set it on channel 2 to mimic having only a master volume control
+		ret = SetCurVolume (interfaceNumber, 2, theValue);
+	}
+
 	if (TRUE == fShouldUpdatePRAM && FALSE == gExpertMode) {				// We do that only if we are on a OS 9 like UI guideline
 		WritePRAMVol (newValue, newValue);
 	}
-
+#if 0
 	// If this is the iSub volume control, don't bother to look for ourselves, which would probably result in an infinite loop anyway.
-	if (getSubType () != 'subL' && getSubType () != 'subR') {
+	if (getSubType () != kIOAudioLevelControlSubTypeLFEVolume && getUsage () == kIOAudioControlUsageOutput) {
 		IOAudioLevelControl *	iSubVolume;
 		IORegistryEntry *		start;
 		IORegistryEntry *		parent;
@@ -212,22 +247,19 @@ void AppleUSBAudioLevelControl::updateUSBValue (SInt32 newValue) {
 		engine = parent->childFromPath ("AppleUSBAudioEngine", gIOServicePlane);
 		FailIf (NULL == engine, Exit);
 
-		if (1 == channelNumber) {
-			iSubVolume = (IOAudioLevelControl *)FindEntryByNameAndProperty (engine, "AppleUSBAudioLevelControl", kIOAudioControlSubTypeKey, 'subL');
-		} else if (2 == channelNumber) {
-			iSubVolume = (IOAudioLevelControl *)FindEntryByNameAndProperty (engine, "AppleUSBAudioLevelControl", kIOAudioControlSubTypeKey, 'subR');
-		}
+		iSubVolume = (IOAudioLevelControl *)FindEntryByNameAndProperty (engine, "AppleUSBAudioLevelControl", kIOAudioControlSubTypeKey, kIOAudioLevelControlSubTypeLFEVolume);
 
 		engine->release ();
 
-		FailIf (NULL == iSubVolume, Exit);
-		newiSubVolume = ((newValue * kiSubMaxVolume) / getMaxValue ()) * kiSubVolumePercent / 100;
-		debugIOLog ("Setting the iSub volume control\n");
-		iSubVolume->setValue (newiSubVolume);
-		iSubVolume->release ();
+		if (NULL != iSubVolume) {
+			newiSubVolume = ((newValue * kiSubMaxVolume) / getMaxValue ()) * kiSubVolumePercent / 100;
+			debugIOLog ("Setting the iSub volume control\n");
+			iSubVolume->setValue (newiSubVolume);
+			iSubVolume->release ();
+		}
 	}
-
-Exit:
+#endif
+//Exit:
     if (ret != kIOReturnSuccess) {
         debug4IOLog ("++AppleUSBAudioLevelContol:updateUSBValue () - set current value for %d:%d failed: 0x%X\n", controlSelector, channelNumber, ret);
     }
@@ -246,7 +278,6 @@ SInt16 AppleUSBAudioLevelControl::GetCurVolume (UInt8 interfaceNumber, UInt8 cha
     devReq.wLength = 2;
     devReq.pData = &theVolume;
 
-//    *error = audioDevice->deviceRequest (&devReq);
 	*error = usbDeviceRequest (&devReq, callerRefCon);
     FailIf (kIOReturnSuccess != *error, Error);
 
@@ -331,23 +362,44 @@ IOReturn AppleUSBAudioLevelControl::SetCurVolume (UInt8 interfaceNumber, UInt8 c
     devReq.wLength = 2;
     devReq.pData = &volume;
 
+	FailIf ((TRUE == isInactive()), DeviceInactive);  	// In case we've been unplugged during sleep
 	error = usbDeviceRequest (&devReq, callerRefCon);
     FailIf (kIOReturnSuccess != error, Exit);
 
 Exit:
     return error;
+	
+DeviceInactive:
+	debugIOLog("AppleUSBAudioLevelControl::SetCurVolume ERROR attempt to send a device request to and inactive device\n");
+	error = kIOReturnError;
+	goto Exit;
 }
 
 void AppleUSBAudioLevelControl::updateValueCallback (void *arg1, void *arg2) {
     AppleUSBAudioLevelControl 		*levelControl;
     SInt32							value;
+	UInt32							subType;
 
     debug3IOLog ("+AppleUSBAudioLevelControl::updateValueCallback (%p, %p)\n", (UInt32*)arg1, (UInt32*)arg2);
     levelControl = OSDynamicCast (AppleUSBAudioLevelControl, (OSObject*)arg1);
     value = (SInt32)arg2;
 
     if (levelControl) {
-        levelControl->updateUSBValue (value);
+		subType = levelControl->getSubType ();
+		
+		if (kIOAudioLevelControlSubTypePRAMVolume == subType) {
+			UInt8 						curPRAMVol;
+			IODTPlatformExpert * 		platform = NULL;
+
+			platform = OSDynamicCast (IODTPlatformExpert, getPlatform ());
+			if (NULL != platform) {
+				platform->readXPRAM ((IOByteCount)kPRamVolumeAddr, &curPRAMVol, (IOByteCount)1);
+				curPRAMVol = (curPRAMVol & 0xF8) | value;
+				platform->writeXPRAM ((IOByteCount)kPRamVolumeAddr, &curPRAMVol, (IOByteCount)1);
+			}
+		} else {
+			levelControl->updateUSBValue (value);
+		}
     }
 
     debug3IOLog ("-AppleUSBAudioLevelControl::updateValueCallback (%p, %p)\n", (UInt32*)arg1, (UInt32*)arg2);
@@ -370,7 +422,7 @@ IOFixed AppleUSBAudioLevelControl::ConvertUSBVolumeTodB (SInt16 volume) {
 
 	return dBVolumeFixed;
 }
-
+#if 0
 IORegistryEntry * AppleUSBAudioLevelControl::FindEntryByNameAndProperty (const IORegistryEntry * start, const char * name, const char * key, UInt32 value) {
 	OSIterator				*iterator;
 	IORegistryEntry			*theEntry;
@@ -400,7 +452,7 @@ Exit:
 	}
 	return theEntry;
 }
-
+#endif
 #pragma mark +PRAM VOLUME
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //	Calculates the PRAM volume value for stereo volume.
@@ -450,7 +502,7 @@ void AppleUSBAudioLevelControl::WritePRAMVol (SInt32 leftVol, SInt32 rightVol) {
 		if (pramVolume != (curPRAMVol & 0x07)) {
 			// clear bottom 3 bits of volume control byte from PRAM low memory image
 			curPRAMVol = (curPRAMVol & 0xF8) | pramVolume;
-            debug2IOLog ("AppleUSBTrinityAudioDevice::WritePRAMVol curPRAMVol=0x%x\n", curPRAMVol);
+            debug2IOLog ("AppleUSBAudioLevelControl::WritePRAMVol curPRAMVol=0x%x\n", curPRAMVol);
 
 			// write out the volume control byte to PRAM
 			platform->writeXPRAM ((IOByteCount)kPRamVolumeAddr, &curPRAMVol, (IOByteCount) 1);
