@@ -31,6 +31,7 @@
 
 #include <libkern/c++/OSArray.h>
 
+#include <IOKit/audio/IOAudioTypes.h>
 #include "USBAudioObject.h"
 #include "AppleUSBAudioCommon.h"
 
@@ -229,6 +230,18 @@ Exit:
 	return theAltInterface;
 }
 
+UInt32 USBAudioConfigObject::GetAC3BSID (UInt8 interfaceNum, UInt8 altInterfaceNum) {
+	USBAudioStreamObject * 				thisStream;
+	UInt32								bmBSID;
+	
+	bmBSID = 0;
+	thisStream = GetStreamObject (interfaceNum, altInterfaceNum);
+	if (thisStream)
+		bmBSID = thisStream->GetAC3BSID ();
+	
+	return bmBSID;
+}
+
 UInt8 USBAudioConfigObject::GetFeatureUnitIDConnectedToOutputTerminal (UInt8 interfaceNum, UInt8 altInterfaceNum, UInt8 outputTerminalID) {
     USBAudioControlObject *				thisControl;
     UInt8								featureUnitID;
@@ -575,6 +588,18 @@ Exit:
 	return lowSampleRate;
 }
 
+UInt16 USBAudioConfigObject::GetMaxBitRate (UInt8 interfaceNum, UInt8 altInterfaceNum) {
+	USBAudioStreamObject * 				thisStream;
+	UInt16								maxBitRate;
+	
+	maxBitRate = 0;
+	thisStream = GetStreamObject (interfaceNum, altInterfaceNum);
+	if (thisStream)
+		maxBitRate = thisStream->GetMaxBitRate ();
+	
+	return maxBitRate;
+}
+
 UInt8 USBAudioConfigObject::GetNumAltStreamInterfaces (UInt8 interfaceNum) {
     USBAudioStreamObject * 				thisStream;
     UInt16								numAltInterfaces;
@@ -676,6 +701,18 @@ UInt8 USBAudioConfigObject::GetNumStreamInterfaces (void) {
     return numInterfaces;
 }
 
+UInt16 USBAudioConfigObject::GetSamplesPerFrame (UInt8 interfaceNum, UInt8 altInterfaceNum) {
+	USBAudioStreamObject * 				thisStream;
+	UInt16								samplesPerFrame;
+	
+	samplesPerFrame = 0;
+	thisStream = GetStreamObject (interfaceNum, altInterfaceNum);
+	if (thisStream)
+		samplesPerFrame = thisStream->GetMaxBitRate ();
+	
+	return samplesPerFrame;
+}
+
 UInt32 * USBAudioConfigObject::GetSampleRates (UInt8 interfaceNum, UInt8 altInterfaceNum) {
     USBAudioStreamObject * 				thisStream;
     UInt32 *							sampleRates;
@@ -1764,6 +1801,7 @@ UInt8 USBAudioStreamObject::GetIsocEndpointSyncType (UInt8 address) {
 
 USBInterfaceDescriptorPtr USBAudioStreamObject::ParseASInterfaceDescriptor (USBInterfaceDescriptorPtr theInterfacePtr, UInt8 const currentInterface) {
     UInt32								indx;
+	UInt16								wFormatTag;
     Boolean								done;
     USBEndpointObject *					theEndpoint;
 
@@ -1816,8 +1854,8 @@ USBInterfaceDescriptorPtr USBAudioStreamObject::ParseASInterfaceDescriptor (USBI
 							break;
 						case FORMAT_TYPE_II:
 							debugIOLog ("in FORMAT_TYPE_II in FORMAT_TYPE\n");
-							maxBitRate = ((ASFormatTypeIIDescriptorPtr)theInterfacePtr)->wMaxBitRate;
-							samplesPerFrame = ((ASFormatTypeIIDescriptorPtr)theInterfacePtr)->wSamplesPerFrame;
+							maxBitRate = USBToHostWord (((ASFormatTypeIIDescriptorPtr)theInterfacePtr)->wMaxBitRate);
+							samplesPerFrame = USBToHostWord (((ASFormatTypeIIDescriptorPtr)theInterfacePtr)->wSamplesPerFrame);
 							numSampleFreqs = ((ASFormatTypeIIDescriptorPtr)theInterfacePtr)->bSamFreqType;
 							if (0 != numSampleFreqs) {
 								debugIOLog ("device has a discrete number of sample rates\n");
@@ -1843,6 +1881,32 @@ USBInterfaceDescriptorPtr USBAudioStreamObject::ParseASInterfaceDescriptor (USBI
 							debugIOLog ("!!!!unknown format type in FORMAT_TYPE!!!!\n");
 					}
                     theInterfacePtr = (USBInterfaceDescriptorPtr)((UInt8 *)theInterfacePtr + theInterfacePtr->bLength);
+                    break;
+				case FORMAT_SPECIFIC:
+                    debugIOLog ("in FORMAT_SPECIFIC in ParseASInterfaceDescriptor\n");
+					wFormatTag = ((ASFormatSpecificDescriptorHeaderPtr)theInterfacePtr)->wFormatTag[1] << 8 | ((ASFormatSpecificDescriptorHeaderPtr)theInterfacePtr)->wFormatTag[0];
+					switch (wFormatTag) {
+						case MPEG:
+							debugIOLog ("in FORMAT_SPECIFIC in MPEG\n");
+							bmMPEGCapabilities = ((ASMPEGFormatSpecificDescriptorPtr)theInterfacePtr)->bmMPEGCapabilities[1] << 8 |
+												 ((ASMPEGFormatSpecificDescriptorPtr)theInterfacePtr)->bmMPEGCapabilities[0];
+							bmMPEGFeatures = ((ASMPEGFormatSpecificDescriptorPtr)theInterfacePtr)->bmMPEGFeatures;
+							break;
+						case AC3:
+							debugIOLog ("in FORMAT_SPECIFIC in AC3\n");
+							bmAC3BSID = ((ASAC3FormatSpecificDescriptorPtr)theInterfacePtr)->bmBSID[3] << 24 |
+										((ASAC3FormatSpecificDescriptorPtr)theInterfacePtr)->bmBSID[2] << 16 |
+										((ASAC3FormatSpecificDescriptorPtr)theInterfacePtr)->bmBSID[1] << 8 |
+										((ASAC3FormatSpecificDescriptorPtr)theInterfacePtr)->bmBSID[0];
+							IOLog ("bmAC3BSID = 0x%x\n", bmAC3BSID);
+							bmAC3Features = ((ASAC3FormatSpecificDescriptorPtr)theInterfacePtr)->bmAC3Features;
+							IOLog ("bmAC3Features = 0x%x\n", bmAC3Features);
+							break;
+						default:
+							debug2IOLog ("!!!!unknown format type 0x%x in FORMAT_SPECIFIC!!!!\n", wFormatTag);
+							break;
+					}
+                    theInterfacePtr = (USBInterfaceDescriptorPtr)((UInt8 *)theInterfacePtr + theInterfacePtr->bLength);
                     break;
                 default:
                     debugIOLog ("in default in ParseASInterfaceDescriptor\n");
