@@ -81,7 +81,8 @@ enum {
 	SUBCLASS_UNDEFINED						= 0x00,
     AUDIOCONTROL							= 0x01,
     AUDIOSTREAMING							= 0x02,
-	MIDISTREAMING							= 0x03
+	MIDISTREAMING							= 0x03,
+	VENDOR_SPECIFIC							= 0xff
 };
 
 enum {
@@ -206,6 +207,7 @@ enum {
 	PITCH_CONTROL							= 0x02
 };
 
+#if 0
 // Input terminal types
 enum {
     INPUT_UNDEFINED							= 0x0200,
@@ -282,6 +284,7 @@ enum {
     EMBEDDED_MULTITRACK_RECORDER			= 0x0712,
     EMBEDDED_SYNTHESIZER					= 0x0713
 };
+#endif
 
 #define USB_AUDIO_IS_FUNCTION(subtype)	((subtype >= MIXER_UNIT) && (subtype <= EXTENSION_UNIT))
 #define USB_AUDIO_IS_TERMINAL(subtype)	((subtype == INPUT_TERMINAL) || (subtype == OUTPUT_TERMINAL))
@@ -493,6 +496,31 @@ typedef struct ASInterfaceDescriptor {
     UInt8									wFormatTag[2];	// because it's not PPC aligned when declared as UInt16
 } ASInterfaceDescriptor, *ASInterfaceDescriptorPtr;
 
+typedef struct ASFormatSpecificDescriptorHeader {
+    UInt8									bLength;
+    UInt8									bDescriptorType;
+    UInt8									bDescriptorSubtype;
+    UInt8									wFormatTag[2];	// because it's not PPC aligned when declared as UInt16
+} ASFormatSpecificDescriptorHeader, *ASFormatSpecificDescriptorHeaderPtr;
+
+typedef struct ASMPEGFormatSpecificDescriptor {
+    UInt8									bLength;
+    UInt8									bDescriptorType;
+    UInt8									bDescriptorSubtype;
+    UInt8									wFormatTag[2];	// because it's not PPC aligned when declared as UInt16
+	UInt8									bmMPEGCapabilities[2];	// because it's not PPC aligned when declared as UInt16
+	UInt8									bmMPEGFeatures;
+} ASMPEGFormatSpecificDescriptor, *ASMPEGFormatSpecificDescriptorPtr;
+
+typedef struct ASAC3FormatSpecificDescriptor {
+    UInt8									bLength;
+    UInt8									bDescriptorType;
+    UInt8									bDescriptorSubtype;
+    UInt8									wFormatTag[2];	// because it's not PPC aligned when declared as UInt16
+	UInt8									bmBSID[4];		// because it's not PPC aligned when declared as UInt32
+	UInt8									bmAC3Features;
+} ASAC3FormatSpecificDescriptor, *ASAC3FormatSpecificDescriptorPtr;
+
 typedef struct ASEndpointDescriptor {
     UInt8									bLength;
     UInt8									bDescriptorType;
@@ -753,9 +781,13 @@ private:
     USBCSASIsocADEndpointObject *	theIsocEndpointObject;
 
     UInt32 *						sampleFreqs;
+	UInt32							bmAC3BSID;
+	UInt16							bmMPEGCapabilities;
     UInt16							formatTag;
 	UInt16							maxBitRate;
 	UInt16							samplesPerFrame;
+	UInt8							bmMPEGFeatures;
+	UInt8							bmAC3Features;
     UInt8							interfaceNumber;
     UInt8							alternateSetting;
     UInt8							numEndpoints;
@@ -788,6 +820,7 @@ public:
 
     USBInterfaceDescriptorPtr		ParseASInterfaceDescriptor (USBInterfaceDescriptorPtr theInterfacePtr, UInt8 const currentInterface);
 
+	UInt32							GetAC3BSID (void) {return bmAC3BSID;}
     UInt8							GetAltInterfaceNum (void) {return alternateSetting;}
 	UInt8							GetDelay (void) {return delay;}
 //	UInt8							GetEndpointAddress (void) {if (theEndpointObject) return theEndpointObject->GetAddress (); else return 0;}
@@ -804,9 +837,11 @@ public:
 	UInt8							GetIsocEndpointDirection (UInt8 index);
 	UInt8							GetIsocEndpointSyncType (UInt8 address);
 //	UInt8							GetMaxPacketSize (void) {if (theEndpointObject) return theEndpointObject->GetMaxPacketSize (); else return 0;}
+	UInt16							GetMaxBitRate (void) {return maxBitRate;}
     UInt8							GetNumChannels (void) {return numChannels;}
 	UInt8							GetNumEndpoints (void) {return numEndpoints;}
     UInt8							GetNumSampleRates (void) {return numSampleFreqs;}
+	UInt16							GetSamplesPerFrame (void) {return samplesPerFrame;}
     UInt8							GetSampleSize (void) {return bitResolution;}
     UInt8							GetSubframeSize (void) {return subframeSize;}
     UInt32 *						GetSampleRates (void) {return sampleFreqs;}
@@ -839,6 +874,7 @@ public:
 	UInt8							FindNextAltInterfaceWithSampleSize (UInt8 interfaceNum, UInt8 startingAltInterface, UInt8 sampleSizeRequested);
 	UInt8							FindNextAltInterfaceWithSampleRate (UInt8 interfaceNum, UInt8 startingAltInterface, UInt32 sampleRateRequested);
 	UInt8							FindAltInterfaceWithSettings (UInt8 interfaceNum, UInt8 numChannels, UInt8 sampleSize, UInt32 sampleRate = 0);
+	UInt32							GetAC3BSID (UInt8 interfaceNum, UInt8 altInterfaceNum);
 	UInt8							GetFeatureUnitIDConnectedToOutputTerminal (UInt8 interfaceNum, UInt8 altInterfaceNum, UInt8 outputTerminalID);
     UInt8							GetFirstStreamInterfaceNum (void);
     UInt8							GetControlInterfaceNum (void);
@@ -860,6 +896,7 @@ public:
     UInt8							GetInterfaceClass (UInt8 interfaceNum, UInt8 altInterfaceNum);
     UInt8							GetInterfaceSubClass (UInt8 interfaceNum, UInt8 altInterfaceNum);
 	UInt32							GetLowestSampleRate (UInt8 interfaceNum, UInt8 altInterfaceNum);
+	UInt16							GetMaxBitRate (UInt8 interfaceNum, UInt8 altInterfaceNum);
     UInt8							GetNumAltStreamInterfaces (UInt8 interfaceNum);
     UInt8							GetNumChannels (UInt8 interfaceNum, UInt8 altInterfaceNum);
     UInt8							GetNumSampleRates (UInt8 interfaceNum, UInt8 altInterfaceNum);
@@ -868,6 +905,7 @@ public:
 	UInt8							GetNumSelectorSources (UInt8 interfaceNum, UInt8 altInterfaceNum, UInt8 unitID);
     UInt8							GetNumStreamInterfaces (void);
 	UInt16							GetOutputTerminalType (UInt8 interfaceNum, UInt8 altInterfaceNum, UInt8 terminalID);
+	UInt16							GetSamplesPerFrame (UInt8 interfaceNum, UInt8 altInterfaceNum);
     UInt32 *						GetSampleRates (UInt8 interfaceNum, UInt8 altInterfaceNum);
     UInt8							GetSampleSize (UInt8 interfaceNum, UInt8 altInterfaceNum);
 	UInt8 *							GetSelectorSources (UInt8 interfaceNum, UInt8 altInterfaceNum, UInt8 unitID);
